@@ -1,6 +1,6 @@
 /*
  *  Methods for encryption/security mechanisms for cryptmount
- *  (C)Copyright 2007-2012, RW Penney
+ *  (C)Copyright 2007-2013, RW Penney
  */
 
 /*
@@ -37,6 +37,9 @@
 #  include "cmtesting.h"
 #endif
 
+/*! \addtogroup keymgrs
+ *  @{ */
+
 
 typedef struct {
     unsigned fversion;
@@ -346,7 +349,7 @@ static int kmblti_put_key(bound_tgtdefn_t *boundtgt,
     }
 
     /* Generate salt & record in key-file: */
-    get_randkey(salt, sizeof(salt));
+    cm_generate_key(salt, sizeof(salt));
     wr_errs += cm_fwrite((const void*)salt, sizeof(salt), fp_key);
 
     /* Augment key with simple checksum: */
@@ -511,6 +514,8 @@ keymanager_t *kmblti_gethandle(void)
     return &keymgr_blti;
 }
 
+/**  @} */
+
 /*
- *  (C)Copyright 2007-2012, RW Penney
+ *  (C)Copyright 2007-2013, RW Penney
  */
