@@ -1,6 +1,6 @@
 /*
  *  Methods for encryption/security mechanisms for cryptmount
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
 
 /*
@@ -38,6 +38,8 @@
 #  include "cmtesting.h"
 #endif
 
+/*! \addtogroup keymgrs
+ *  @{ */
 
 
 /*
@@ -590,7 +592,7 @@ static int kmgcry_put_key(bound_tgtdefn_t *boundtgt,
     wr_errs += cm_fwrite((const void*)hbuff, (size_t)4, fp_key);
 
     /* Generate salt & record in keyfile: */
-    get_randkey(salt, sizeof(salt));
+    cm_generate_key(salt, sizeof(salt));
     wr_errs += cm_fwrite((const void*)salt, sizeof(salt), fp_key);
 
     /* Augment key with simple checksum: */
@@ -742,7 +744,7 @@ static int kmgcryossl_put_key(bound_tgtdefn_t *boundtgt,
     wr_errs += cm_fwrite((const void*)kmgcryossl_magstr, kmgcryossl_maglen, fp_key);
 
     /* Generate salt & record in keyfile: */
-    get_randkey(salt, sizeof(salt));
+    cm_generate_key(salt, sizeof(salt));
     wr_errs += cm_fwrite((const void*)salt, sizeof(salt), fp_key);
 
     /* Pad key-data with end-marker: */
@@ -882,6 +884,8 @@ keymanager_t *kmgcry_gethandle()
 
 #endif  /* !AS_MODULE */
 
+/**  @} */
+
 /*
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
